from datetime import datetime
from zla_utilities import \
    utility as ut

def get_sql_1(partnolist):
    sql_1 = 'orders.partno in {}'
    sel_data = sql_1.format(ut.sel_insql(partnolist))
    return sel_data


def get_sql_2(customerlist):
    sql_1 = 'ws.customerid in {}'
    if customerlist is not None:
        cus_data = sql_1.format(ut.sel_insql(customerlist))
    else:
        cus_data = None
    return cus_data


def get_sql_3(mlist):
    sql_base = 'orders.del1stdate '
    if all(x is None for x in mlist):
        _sql = sql_base + '>= DATE_SUB(now(), INTERVAL 48 MONTH)'
    elif all(type(x) is str for x in mlist):
        _subsql = convertor_bothdate(mlist)
        _sql = sql_base + _subsql
    else:
        _subsql = convertor_singledate(mlist)
        _sql = sql_base + _subsql
    return _sql


def convertor_bothdate(dlist):
    _start = datetime.strptime(dlist[0], '%Y-%m-%d')
    _end = datetime.strptime(dlist[1], '%Y-%m-%d')
    if _start > _end:
        print('Not allow start_date less than end_date !!')
        _sub = 'between "x" and "y"'
    else:
        _sub = 'between "{}" and "{}"'.format(dlist[0], dlist[1])

    return _sub


def convertor_singledate(dlist):
    if dlist[0] is not None:
        _sub = '>= "{}"'.format(dlist[0])
    else:
        _sub = 'between DATE_SUB("{}", INTERVAL 48 MONTH) and "{}"'.format(dlist[1], dlist[1])
    return _sub


def merge_wherecondition(sql_sentense):
    _where = [' (' + _sen + ') ' for _sen in sql_sentense if _sen is not None]
    return "and".join(_where)


def finishing_sql(_where):
    _sql = """select orders.partno, ws.customerid, ws.shipto ,ws.sotype,ws.sorg,ws.distch, ws.orderdate, orders.del1stdate,sum(orders.qty) as qty
            from partssaleorder as ws inner join orderdetails as orders on ws.saleorder = orders.saleorder
            where"""
    _groupby = " group by orders.partno, ws.customerid, ws.shipto , ws.sotype, ws.sorg, ws.distch, ws.orderdate,orders.del1stdate"
    msql = _sql + _where + _groupby
    return msql


from datetime import datetime
from zla_utilities import \
    utility as ut

def get_sql_1(partnolist):
    sql_1 = 'rsdet.partno in {}'
    sql_2 = 'servdet.partno in {}'
    sel_rs = sql_1.format(ut.sel_insql(partnolist))
    sel_serv = sql_2.format(ut.sel_insql(partnolist))
    return sel_rs,sel_serv


def get_sql_2(customerlist):
    sql_1 = """rs.customerid in (select idcustomerlistkad from customerlistkad where idcustomersap in {})"""
    sql_2 = """serv.customerid in (select idcustomerlistkad from customerlistkad where idcustomersap in {})"""
    if customerlist is not None:
        rscus_data = sql_1.format(ut.sel_insql(customerlist))
        servcus_data = sql_2.format(ut.sel_insql(customerlist))
    else:
        rscus_data = None
        servcus_data = None
    return rscus_data,servcus_data


def get_sql_3(mlist):
    sql_base1 = 'rs.rsdate '
    sql_base2 = 'serv.docdate '
    if all(x is None for x in mlist):
        _rssql = sql_base1 + '>= DATE_SUB(now(), INTERVAL 48 MONTH)'
        _servsql = sql_base2 + '>= DATE_SUB(now(), INTERVAL 48 MONTH)'
    elif all(type(x) is str for x in mlist):
        _subsql = convertor_bothdate(mlist)
        _rssql = sql_base1 + _subsql
        _servsql = sql_base2 + _subsql
    else:
        _subsql = convertor_singledate(mlist)
        _rssql = sql_base1 + _subsql
        _servsql = sql_base2 + _subsql
    return _rssql,_servsql


def convertor_bothdate(dlist):
    _start = datetime.strptime(dlist[0], '%Y-%m-%d')
    _end = datetime.strptime(dlist[1], '%Y-%m-%d')
    if _start > _end:
        print('Not allow start_date less than end_date !!')
        _sub = 'between "x" and "y"'
    else:
        _sub = 'between "{}" and "{}"'.format(dlist[0], dlist[1])

    return _sub


def convertor_singledate(dlist):
    if dlist[0] is not None:
        _sub = '>= "{}"'.format(dlist[0])
    else:
        _sub = 'between DATE_SUB("{}", INTERVAL 48 MONTH) and "{}"'.format(dlist[1], dlist[1])
    return _sub


def merge_wherecondition(sql_sentense):
    _where = [' (' + _sen + ') ' for _sen in sql_sentense if _sen is not None]
    return "and".join(_where)


def finishing_sql(_rswhere,_servwhere):
    _sqlrs = """select rs.rsdate as date,kad.idcustomersap as customerid, rsdet.qty
            from rsorder as rs inner join rsorderdetails as rsdet on rs.idrsorder = rsdet.idrsorder
            inner join customerlistkad as kad on kad.idcustomerlistkad = rs.customerid
            where"""
    _sqlserv = """select serv.docdate as date,kad.idcustomersap as customerid, servdet.qty
                from servicedata as serv inner join servicedetails as servdet on serv.idservicedata = servdet.idservicedata
                inner join customerlistkad as kad on kad.idcustomerlistkad = serv.customerid
                where"""
    _groupbyrs = " group by rs.rsdate,kad.idcustomersap"
    _groupbyserv = " group by serv.docdate,kad.idcustomersap"
    rs_msql = _sqlrs + _rswhere + _groupbyrs
    sev_msql = _sqlserv + _servwhere + _groupbyserv
    return rs_msql,sev_msql


import pandas as pd
from zla_utilities import \
    db_connector as dbcon, utility as ut
import warnings
warnings.filterwarnings("ignore")

def get_data(msql):
    db = 'partsdatabase'
    df = dbcon.opendata(msql, db)
    return df

def merge_data(df1,df2):
    df3 = df1.set_index(['date', 'customerid']).join(df2.set_index(['date', 'customerid']), lsuffix='_frnt',rsuffix='_serv', how='outer').fillna(0)
    df3['qty'] = df3['qty_frnt'] + df3['qty_serv']
    df3 = df3[['qty']].reset_index()
    return df3

def to_number(df, _by):
    _data = df[['date', 'qty']].groupby('date').sum()
    _data.index = pd.to_datetime(_data.index, format='%Y-%m-%d')
    _data = ut.full_daterange(_data, by=_by)
    return _data


from .master_sql import *
from .rs_data_sql import *
from .rs_method import *
from zla_general import \
    vendor_finding_bymat as vmat, \
    header_ecnbysap_bymat as ecnmat

class get_rs(object):
    def __init__(self, partno, start=None, end=None, customer=None, parent=False):
        """
        For getting SKC retail data as your requirment
        :param partno: part number is needed.
        :param start: start date (option)
        :param end: end date (option)
        :param customer: customer code by sap only (option)
        :param parent: if True data will be get all of branch data together (default is False)
        """
        # Make character sentense
        self.partno = partno
        self.allcustomer = customer_parentable(customer, parent)
        self.dictchain = ecnmat.get_chaindictionary(partno)
        self.master = get_partsmaster(partno)
        self.character = get_partscharacter(partno)
        self.purchasing = get_partspurchasing(partno)
        self.vendor, self.cost, self.mapping = vmat.get_vendor_mat(partno, self.purchasing.get('spt'))

        # Make sql_sentense
        sql_frntpartno, sql_servpartno = get_sql_1(self.dictchain.keys())
        sql_frntcustomer, sql_servcustomer = get_sql_2(self.allcustomer)
        sql_frnttimerange, sql_servtimerange = get_sql_3([start, end])
        list_frntsql = [sql_frntpartno, sql_frntcustomer, sql_frnttimerange]
        list_servsql = [sql_servpartno, sql_servcustomer, sql_servtimerange]
        where_frntsql = merge_wherecondition(list_frntsql)
        where_servsql = merge_wherecondition(list_servsql)
        frnt_msql,serv_msql = finishing_sql(where_frntsql,where_servsql)
        self.frntdata = History(get_data(frnt_msql))
        self.servdata = History(get_data(serv_msql))
        self.rsdata = History(merge_data(self.frntdata.raw,self.servdata.raw))


class History(pd.DataFrame):
    def __init__(self,_data):
        super(History, self).__init__()
        self.raw = _data

    def history(self,_by='days'):
        """
        generate SKC retails sale data as your requirement
        :param _by:
        - 'days'(default) is shown for dialy data
        - 'weeks' is shown for weekly data
        - 'months' is shown for monthly data
        - 'years' is shown for annual data}
        :return:
        Dataframe with date index
        """
        return to_number(self.raw,_by)



import pandas as pd
from zla_utilities import \
    db_connector as dbcon, utility as ut
import warnings
warnings.filterwarnings("ignore")

def get_data(msql):
    db = 'partsdatabase'
    df = dbcon.opendata(msql, db)
    return df


def to_number(df, _by):
    _data = df[['deldate', 'qty']].groupby('deldate').sum()
    _data.index = pd.to_datetime(_data.index, format='%Y-%m-%d')
    _data = ut.full_daterange(_data, by=_by)
    return _data



import numpy as np
from zla_utilities import \
    db_connector as dbcon
import warnings
warnings.filterwarnings("ignore")

def customer_parentable(customer,parent):
    if customer is not None:
        ans = np.array([customer])
        if parent == True:
            db = 'partsdatabase'
            msql = "select customerid from customerlistsap where parent = '{}'".format(customer)
            cus_list = dbcon.opendata(msql,db)['customerid'].values
            if len(cus_list) > 0:
                ans = cus_list
    else:
        ans = None
    return ans

def get_partsmaster(_partno):
    db = 'partsdatabase'
    msql =  "select partname,mrpcn,pdt+grt as LT, weight,MS,submodel from partsmaster where partno = '{}'".format(_partno)
    _ans = dbcon.opendata(msql,db).loc[0].to_dict()
    return _ans

def get_partscharacter(_partno):
    db = 'partsdatabase'
    msql =  "select groupid, frequency ,notice, controller,charc as chars,createon from partscharacter where partno = '{}'".format(_partno)
    _ans = dbcon.opendata(msql,db).loc[0].to_dict()
    return _ans

def get_partspurchasing(_partno):
    db = 'partsdatabase'
    msql =  "select pgr,spt,proctype from partspurchasing where partno = '{}'".format(_partno)
    _ans = dbcon.opendata(msql,db).loc[0].to_dict()
    return _ans


from .master_sql import *
from .ws_data_sql import *
from .ws_method import *
from zla_general import \
    vendor_finding_bymat as vmat, \
    header_ecnbysap_bymat as ecnmat, \
    get_newdeldate_bydf as newdel


class get_ws(object):
    def __init__(self, partno, start=None, end=None, customer=None, parent=False):
        """
        For getting SKC whole data as your requirment
        :param partno: part number is needed.
        :param start: start date (option)
        :param end: end date (option)
        :param customer: customer code by sap only (option)
        :param parent: if True data will be get all of branch data together (default is False)
        """
        # Make character sentense
        self.partno = partno
        self.allcustomer = customer_parentable(customer, parent)
        self.dictchain = ecnmat.get_chaindictionary(partno)
        self.master = get_partsmaster(partno)
        self.character = get_partscharacter(partno)
        self.purchasing = get_partspurchasing(partno)
        self.vendor, self.cost, self.mapping = vmat.get_vendor_mat(partno, self.purchasing.get('spt'))

        # Make sql_sentense
        sql_nontapa = "orders.itemcat <> 'TAPA'"
        sql_partno = get_sql_1(self.dictchain.keys())
        sql_customer = get_sql_2(self.allcustomer)
        sql_timerange = get_sql_3([start, end])
        list_sql = [sql_nontapa, sql_partno, sql_customer, sql_timerange]
        where_sql = merge_wherecondition(list_sql)
        msql = finishing_sql(where_sql)
        _original = get_data(msql)
        _original['deldate'] = _original['del1stdate']
        self.original_data = History(_original)
        self.newdel_data = History(newdel.generate_deldate(_original))



class History(pd.DataFrame):
    def __init__(self,_data):
        super(History, self).__init__()
        self.raw = _data

    def history(self,_filter ='all',_by='days'):
        """
        generate SKC whole sale data as your requirement
        :param _filter:
        - 'all' (default) is overall volume
        - 'dom_all' is domestic all'
        - 'dom_ndist' is domestic for non distributor,
        - 'dom_dist' is domestic for distributor only,
        - 'exp_all' is export all
        - 'exp_nkbt' is export for SKC territory only
        - 'exp_kbt' is export for non territory}
        :param _by:
        - 'days'(default) is shown for dialy data
        - 'weeks' is shown for weekly data
        - 'months' is shown for monthly data
        - 'years' is shown for annual data}
        :return:
        Dataframe with date index
        """
        _his = self.raw.copy()
        if _filter == 'all':
            _ans = to_number(_his,_by)
        elif _filter == 'dom_all':
            filter_1 = (self.raw['sorg'] == 'DY10')
            _ans = to_number(_his[filter_1],_by)
        elif _filter == 'dom_ndist':
            filter_1 = (self.raw['sorg'] == 'DY10')
            filter_2 = (self.raw['distch'] != '20')
            _ans = to_number(_his[filter_1 & filter_2], _by)
        elif _filter == 'dom_dist':
            filter_1 = (self.raw['sorg'] == 'DY10')
            filter_2 = (self.raw['distch'] == '20')
            _ans = to_number(_his[filter_1 & filter_2], _by)
        elif _filter == 'exp_all':
            filter_1 = (self.raw['sorg'] == 'DY20')
            _ans = to_number(_his[filter_1], _by)
        elif _filter == 'exp_nkbt':
            filter_1 = (self.raw['sorg'] == 'DY20')
            filter_2 = (self.raw['distch'] != '30')
            _ans = to_number(_his[filter_1 & filter_2], _by)
        elif _filter == 'exp_kbt':
            filter_1 = (self.raw['sorg'] == 'DY20')
            filter_2 = (self.raw['distch'] == '30')
            _ans = to_number(_his[filter_1 & filter_2], _by)
        else:
            raise Exception("Sorry, your filter was wrong (allow only : ['all','dom_all','dom_ndist','dom_dist','exp_all','exp_nkbt','exp_kbt'])")
        return _ans